// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of scatter
//
// scatter is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// scatter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <suplib/str.h>

#include "MirrorDatabase.h"

MirrorDatabase::MirrorDatabase( 
  const string& mirrordb,
  const int surfno,
  const int shell
) throw ( Exception ) {

  try {

    char surfno_shell[3];
    sprintf( surfno_shell, "%c%d", 1 == surfno ? 'p' : 'h', shell );

    int error;
    char *temp = str_interp( mirrordb.c_str(), 1, NULL, NULL, &error );
    if ( error )
      throw Exception( string( "error in interpolation of file name: " )
		       + mirrordb );
    string mirrordb_in( temp );
    free( temp );

    RDB rdb( mirrordb_in );
    
    while ( rdb.read( ) ) {

      if ( rdb.getColumn( "mirror" )->getDataString( ) == surfno_shell ) {

	_mirror = rdb.getColumn( "mirror" )->getDataString( );
	_z0     = rdb.getColumn( "z0"     )->getDataDouble( );
	return;

      }

    }    

    throw( Exception( string( "couldn't find mirror " ) 
		      + surfno_shell + "in " + mirrordb_in ) );

  } catch ( RDBErr& re ) {

    re.set_message( "error opening mirror database:  " );
    throw( (Exception)re );

  } catch ( Exception& E ) {

    E.set_message( "error opening mirror database:  " );
    throw( E );

  } catch ( exception& e ) {

    throw( Exception( string("error opening mirror database:  ") + e.what( ) ) );

  } catch ( ... ) {

    throw( Exception( "error opening mirror database:  unexpected exception caught" ) );

  }
}
