#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include <suplib/stats.h>

#include <fitsio.h>


static
void chk_status( int status )
{
    if ( status )
    {
	fits_report_error( stderr, status );
	exit(1);
    }

}

static size_t
read_data_txt( char* file, double** rdata_pp )
{
  char buffer[1024];
  size_t nrecords = 0;
  double *data, *tdata;

  FILE *fp = fopen( file, "r" );

  assert ( NULL != fp );

  /* count number of records */
  while( fgets( buffer, sizeof(buffer), fp ) )
    nrecords++;

  tdata = data = malloc( nrecords * sizeof(double) );
  assert( NULL != data );

  fseek( fp, 0L, SEEK_SET );

  while( fgets( buffer, sizeof(buffer), fp ) )
    *tdata++ = strtod( buffer, NULL );

  fclose( fp );

  *rdata_pp = data;
  return nrecords;
}

static size_t
read_data_fits( char* file, char* col, int flip, double** rdata_pp )
{
    fitsfile *fptr;
    long nrows;
    int status = 0;
    int colnum;
    double *data;

    fits_open_table( &fptr, file, READONLY, &status );

    chk_status( status );

    fits_get_colnum( fptr, 0, col, &colnum, &status );

    chk_status( status );

    fits_get_num_rows( fptr, &nrows, &status );

    chk_status( status );

    data = malloc( nrows * sizeof(double) );

    assert( NULL != data );

    fits_read_col( fptr, TDOUBLE, colnum, 1, 1, nrows, NULL, data, NULL, &status );

    if ( flip )
    {
	size_t i;
	for ( i = 0 ; i < nrows ; i++ )
	    data[i] *= -1;
    }

    chk_status( status );

    *rdata_pp = data;
    return nrows;
}

int
main( int argc, char *argv[] )
{
    char* file1;
    char* file2;
    char* col1;
    char* col2;

    int flip1 = 0;
    int flip2 = 0;
    int nd1, nd2;
    int idx = 0;
    int test;
    double *d1, *d2;
    double dstats[3], q;

  int rv, k, *m;

  if ( argc != 5 )
  {
    fprintf( stderr, "usage: test_smirnov file1 col1 file2 col2\n" );
    return EXIT_FAILURE;
  }

  file1 = argv[++idx];
  col1  = argv[++idx];
  if ( col1[0] == '-' )
  {
      flip1 = 1;
      col1++;
  }

  file2 = argv[++idx];
  col2  = argv[++idx];
  if ( col2[0] == '-' )
  {
      flip2 = 1;
      col2++;
  }

  nd1 = read_data_fits( file1, col1, flip1, &d1 );
  nd2 = read_data_fits( file2, col2, flip2, &d2 );

  rv = stcalc( 1, nd1, nd2, d1, d2, &k, &m, dstats );

  if ( rv )
  {
    fprintf( stderr, "stcalc returned %d\n", rv );
    return EXIT_FAILURE;
  }

  /* we want the two-sided test */
  test = 1;
  rv = gsmirn2( nd1, nd2, test, m, dstats[test-1], &q );

  /* underflow we don't care about */
  if ( rv && rv != 3)
  {
      fprintf( stderr, "gsmirn returned %d\n", rv );
      return EXIT_FAILURE;
  }

  printf( "%.14g\n", q );
  return EXIT_SUCCESS;
}
